==> vmaid-1.8.24.tar.gz: vmaid-1.8.24/src/file.c <==
/*
    Video maid
    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "file.h"
#include "sigfile.h"
#include "sigmain.h"
#include "size.h"
#include "thread.h"
#include "misc/fileio.h"
#include "misc/misc.h"
#include "misc/misc_mdi.h"


/******************************************************************************
*                                                                             *
* ja:ファイル関数群                                                           *
*                                                                             *
******************************************************************************/
/*  ja:ファイルの履歴に加える
    file,ファイル名                                                         */
void
file_set_history (const gchar *file)
{
  gchar *label;
  gint i, count;
  GList *glist;
  GtkWidget *menu_shell, *menu_item;

  menu_shell = gtk_item_factory_get_widget (ifactory_menu, "<main>/File");
  glist = gtk_container_children (GTK_CONTAINER (menu_shell));
  count = g_list_length (glist);
  for (i = 0; i < count - MENUFILE - 1; i++)
    {
      menu_item = g_list_nth_data (glist, i + MENUFILE - 1);
      label = g_object_get_data (G_OBJECT (menu_item), "user_data");
      if (!g_strfilecmp (label, file))
        {
          gtk_container_remove (GTK_CONTAINER (menu_shell), menu_item);
          break;
        }
    }
  g_list_free (glist);
  if (i >= history)
    {
      glist = gtk_container_children (GTK_CONTAINER (menu_shell));
      gtk_container_remove (GTK_CONTAINER (menu_shell),
                        g_list_nth_data (glist, MENUFILE + history + 1 - 3));
      g_list_free (glist);
    }
  if (count <= MENUFILE)
    {
      menu_item = gtk_menu_item_new ();
      gtk_menu_insert (GTK_MENU (menu_shell), menu_item, MENUFILE - 2);
        gtk_widget_show (menu_item);
    }
  label = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
  menu_item = gtk_menu_item_new_with_label (label);
  g_free (label);
  label = g_strdup (file);
  g_signal_connect (G_OBJECT (menu_item), "activate",
                            G_CALLBACK (signal_activate_menu_history), label);
  g_signal_connect (G_OBJECT (menu_item), "destroy",
                            G_CALLBACK (signal_destroy_menu_history), label);
  g_object_set_data (G_OBJECT (menu_item), "user_data", label);
  gtk_menu_insert (GTK_MENU (menu_shell), menu_item, MENUFILE - 1);
  gtk_widget_show (menu_item);
}


/******************************************************************************
*                                                                             *
* ja:ファイル入力関数群                                                       *
*                                                                             *
******************************************************************************/
/*  ja:AVIファイルを開く
     avi,AVI編集構造体
    file,ファイル名                                                         */
static void
file_open_avi (AviEdit     *avi_edit[],
               const gchar *file)
{
  gint i;
  guint32 streams;
  AviEdit *avi_edit_tmp;

  avi_edit[0] = avi_edit[1] = NULL;
  streams = MAX (avi_main_streams (file), 1);
  for (i = 0; i < streams && (!avi_edit[0] || !avi_edit[1]); i++)
    if ((avi_edit_tmp = avi_open (file, i)))
      switch (avi_type (avi_edit_tmp))
        {
          case AVI_STREAM_VIDEO:
            if (!avi_edit[0])
              avi_edit[0] = avi_edit_tmp;
            else
              avi_release (avi_edit_tmp);
              break;
          case AVI_STREAM_AUDIO:
            if (!avi_edit[1])
              avi_edit[1] = avi_edit_tmp;
            else
              avi_release (avi_edit_tmp);
            break;
          default:
            avi_release (avi_edit_tmp);
        }
}


/*  ja:ファイルを開く(独立スレッド)
    file_open,Open情報
     RET,ウインドウ情報                                                     */
VmaidWindow *
file_open_edit (FileOpen *file_open)
{
  gchar *filename;
  gint i, page_num;
#ifdef USE_THREAD
  GThread *id;
#else /* not USE_THREAD */
  gint id = THREAD_ID_STANDARD;
#endif /* not USE_THREAD */
  GtkWidget *xbox, *vbox;
  VmaidWindow *vmaid;

#ifdef USE_THREAD
  id = g_thread_self();
#endif /* USE_THREAD */
  thread_insert (id, THREAD_MODE_OPEN, file_open ? file_open->file : NULL);

  vmaid = g_malloc (sizeof (VmaidWindow));
  vmaid->undo = vmaid->redo = NULL;
  vmaid->rate = default_rate;
  vmaid->scale = 1;
  vmaid->cursor.stream = 0;
  vmaid->cursor.frame = 0;
  vmaid->select.stream = -1;
  vmaid->select.frame = -1;
  vmaid->top = 0;
  vmaid->avi_edit[0] = NULL;
  vmaid->avi_edit[1] = NULL;
  vmaid->timer_id = 0;
  if (!file_open || !file_open->file)
    {
      static gint fcount = 0; /* ja:新規ファイルの数 */

      /* ja:新規 */
      filename = g_strdup_printf ("new%04d.avi", fcount++ % 1000);
    }
  else
    {
      gchar *format = NULL;
      gint j, value, max = 0;

      filename = fileio_get_full_path (file_open->file);
      file_open_avi (vmaid->avi_edit, file_open->file);
      if ((vmaid->avi_edit[0] || vmaid->avi_edit[1]) && file_open->merge)
        {
          gchar *name;

          /* ja:連番 */
          name = g_path_get_basename (file_open->file);
          for (i = g_strlen (name) - 1; i >= 0; i--)
            if ('0' <= name[i] && name[i] <= '9')
              break;
          if (i >= 0)
            {
              gchar *dir, *head, *foot;

              dir = g_path_get_dirname (file_open->file);
              head = g_strdup (name);
              foot = name + i + 1;
              for (j = i - 1; j >= 0; j--)
                if (head[j] < '0' || '9' < head[j])
                  break;
              misc_str_to_val (&value, head + j + 1, 10, FALSE);
              head[j + 1] = '\0';
              format = g_strdup_printf ("%s/%s%%0%dd%s",
                                                    dir, head, i - j, foot);
              g_free (dir);
              g_free (head);
              max = 1;
              while (j < i)
                {
                  max *= 10;
                  j++;
                }
            }
          g_free (name);
        }
      if (format)
        {
          gboolean result = TRUE;
          gchar *file;
          AviEdit *avi_edit_old[2], *avi_edit_tmp[2];

          /* ja:連番 */
          for (i = value - 1; i >= 0; i--)
            {
              /* ja:連番前 */
              file = g_strdup_printf (format, i);
              file_open_avi (avi_edit_tmp, file);
              g_free (file);
              if (!avi_edit_tmp[0] && !avi_edit_tmp[1])
                break;
              for (j = 0; j < 2; j++)
                if (vmaid->avi_edit[j])
                  {
                    avi_edit_old[j] = avi_clone (vmaid->avi_edit[j]);
                    if (!avi_paste (vmaid->avi_edit[j], 0, avi_edit_tmp[j]))
                      result = FALSE;
                    }
                  else
                    {
                      avi_edit_old[j] = NULL;
                      vmaid->avi_edit[j] = avi_edit_tmp[j];
                    }
              if (result)
                {
                  for (j = 0; j < 2; j++)
                    if (avi_edit_old[j])
                      avi_release (avi_edit_old[j]);
                }
              else
                {
                  for (j = 0; j < 2; j++)
                    {
                      if (vmaid->avi_edit[j])
                        avi_release (vmaid->avi_edit[j]);
                      vmaid->avi_edit[j] = avi_edit_old[j];
                    }
                  break;
                }
            }
          for (i = value + 1, result = TRUE; i < max && result; i++)
            {
              /* ja:連番後 */
              file = g_strdup_printf (format, i);
              file_open_avi (avi_edit_tmp, file);
              g_free (file);
              if (!avi_edit_tmp[0] && !avi_edit_tmp[1])
                break;
              for (j = 0; j < 2; j++)
                if (vmaid->avi_edit[j])
                  {
                    avi_edit_old[j] = avi_clone (vmaid->avi_edit[j]);
                    if (!avi_paste (vmaid->avi_edit[j],
                            avi_length (vmaid->avi_edit[j]), avi_edit_tmp[j]))
                      result=FALSE;
                  }
                else
                  {
                    avi_edit_old[j] = NULL;
                    vmaid->avi_edit[j] = avi_edit_tmp[j];
                  }
              if (result)
                {
                  for (j = 0; j < 2; j++)
                    if (avi_edit_old[j])
                      avi_release (avi_edit_old[j]);
                }
              else
                {
                  for (j = 0; j < 2; j++)
                    {
                      if (vmaid->avi_edit[j])
                        avi_release(vmaid->avi_edit[j]);
                      vmaid->avi_edit[j]=avi_edit_old[j];
                    }
                }
            }
        }
      if (!vmaid->avi_edit[0] && !vmaid->avi_edit[1]
                    && avi_file_type (file_open->file) == AVI_TYPE_SCENARIO)
        {
          gchar *scenario = NULL;
          goffset offset;
          AviEdit **avi_edit;
          FileIO *fio;

          /* ja:ファイルを開く */
          fio = fileio_open (file_open->file, FILEIO_ACCESS_READ,
                                FILEIO_SHARE_READ, FILEIO_MODE_OPEN_EXISTING);
          if (!fio)
            {
              g_free (file_open->file);
              g_free (file_open);
              g_free (vmaid);
              return NULL;
            }
          /* ja:読み込み */
          offset = fileio_seek (fio, 0, SEEK_END);
          if (offset == -1 || fileio_seek (fio, 0, FILEIO_SEEK_SET) == -1
                                || !(scenario = g_malloc ((gssize)offset + 1)))
            {
              fileio_close (fio);
              g_free (file_open->file);
              g_free (file_open);
              g_free (vmaid);
              return 0;
            }
          scenario[offset] = '\0';
          if (fileio_read (fio, scenario, (gssize)offset) != offset)
            {
              fileio_close (fio);
              g_free (scenario);
              g_free (file_open->file);
              g_free (file_open);
              g_free (vmaid);
              return 0;
            }
          if (!fileio_close (fio))
            {
              g_free (scenario);
              g_free (file_open->file);
              g_free (file_open);
              g_free (vmaid);
              return 0;
            }
          if ((avi_edit = avi_from_scenario (scenario)))
            for (i = 0; avi_edit[i]; i++)
              switch (avi_type (avi_edit[i]))
                {
                  case AVI_STREAM_VIDEO:
                    if (!vmaid->avi_edit[0])
                      vmaid->avi_edit[0] = avi_edit[i];
                    else
                      avi_release (avi_edit[i]);
                    break;
                  case AVI_STREAM_AUDIO:
                    if (!vmaid->avi_edit[1])
                      vmaid->avi_edit[1] = avi_edit[i];
                    else
                      avi_release (avi_edit[i]);
                    break;
                  default:
                    avi_release(avi_edit[i]);
                }
        }
    }

  if (!thread_idling (id, 100))
    {
      g_free (file_open->file);
      g_free (file_open);
      for (i = 0; i < 2; i++)
        if (vmaid->avi_edit[i])
          avi_release(vmaid->avi_edit[i]);
      g_free (vmaid);
      return NULL;
    }

  if (vmaid->avi_edit[0])
    {
      size_set_scale (default_view, &vmaid->width, &vmaid->height,
                                        avi_get_width (vmaid->avi_edit[0]),
                                        avi_get_height (vmaid->avi_edit[0]));
      vmaid->rate = avi_get_rate (vmaid->avi_edit[0]);
      vmaid->scale = avi_get_scale (vmaid->avi_edit[0]);
    }
  else
    {
      size_set_scale (default_view, &vmaid->width, &vmaid->height, -1, -1);
    }
  if (!vmaid->avi_edit[0] && vmaid->avi_edit[1])
    vmaid->cursor.stream = 1;

#ifdef USE_THREAD
  gdk_threads_enter();
#endif /* USE_THREAD */

  /* ja:表示エリア */
  vmaid->drawing = gtk_drawing_area_new ();
  g_signal_connect (G_OBJECT (vmaid->drawing), "focus-in-event",
                                G_CALLBACK (signal_focus_in), vmaid);
  g_signal_connect (G_OBJECT (vmaid->drawing), "focus-out-event",
                                G_CALLBACK (signal_focus_out), vmaid);
  g_signal_connect (G_OBJECT (vmaid->drawing), "realize",
                                G_CALLBACK (signal_realize), NULL);
  g_signal_connect (G_OBJECT (vmaid->drawing), "configure-event",
                                G_CALLBACK (signal_config), vmaid);
  g_signal_connect (G_OBJECT (vmaid->drawing), "expose-event",
                                G_CALLBACK (signal_expose), vmaid);
  g_signal_connect (G_OBJECT (vmaid->drawing), "button-press-event",
                                G_CALLBACK (signal_button_press_draw), vmaid);
  g_signal_connect (G_OBJECT (vmaid->drawing), "motion-notify-event",
                                G_CALLBACK (signal_motion_notify), vmaid);
  g_signal_connect (G_OBJECT (vmaid->drawing), "button-release-event",
                                G_CALLBACK (signal_button_release), vmaid);
  g_signal_connect (G_OBJECT (vmaid->drawing), "scroll-event",
                                G_CALLBACK (signal_scroll), vmaid);
  g_signal_connect_after (G_OBJECT (vmaid->drawing), "key-press-event",
                                G_CALLBACK (signal_key_press), vmaid);
  g_signal_connect (GTK_OBJECT (vmaid->drawing), "destroy",
                                G_CALLBACK (signal_destroy_draw), vmaid);
  gtk_widget_add_events (vmaid->drawing, GDK_POINTER_MOTION_MASK
                            | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
                            | GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK
                            | GDK_FOCUS_CHANGE_MASK);
  gtk_widget_set_usize (vmaid->drawing,
                vmaid->width * 3 + 4, system_font_height * 2
                + ((vmaid->avi_edit[0] ? 1 : 0) + (vmaid->avi_edit[1] ? 1 : 0))
                                                            * vmaid->height);
  GTK_WIDGET_SET_FLAGS (vmaid->drawing, GTK_CAN_FOCUS);
  /* ja:スクロールバー */
  vmaid->hscroll = gtk_hscrollbar_new
                    (GTK_ADJUSTMENT (gtk_adjustment_new (0, 0, 0, 1, 1, 1)));
  /* ja:ボックス */
  xbox = gtk_hbox_new (FALSE, 0);
  vbox = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), vmaid->drawing, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), vmaid->hscroll, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (xbox), vbox, FALSE, FALSE,0);
  /* ja:表示 */
  page_num = misc_mdi_append_page (MISC_MDI (mdi),
                        xbox, filename, !file_open || !file_open->file, vmaid);
  gtk_widget_show_all (xbox);
  gtk_notebook_set_page (GTK_NOTEBOOK (mdi), page_num);
  gtk_widget_grab_focus (vmaid->drawing);

  /* ja:新規作成ではなく、ファイルが存在する時には履歴に加える */
  if (file_open && history > 0 && fileio_isfile (file_open->file))
    file_set_history (filename);

#ifdef USE_THREAD
  gdk_threads_leave();
#endif /* USE_THREAD */

  if (file_open)
    {
      g_free (file_open->file);
      g_free (file_open);
    }
  g_free (filename);
  thread_delete (id);

  return vmaid;
}


/******************************************************************************
*                                                                             *
* ja:ファイル出力関数群                                                       *
*                                                                             *
******************************************************************************/
static gboolean
file_save_edit_callback (gint percent, gpointer user_data)
{
#ifdef USE_THREAD
  return thread_idling (g_thread_self (), percent);
#else /* not USE_THREAD */
  return thread_idling (THREAD_ID_STANDARD, percent);
#endif /* not USE_THREAD */
}


/*  ja:AVIファイルを保存する(独立スレッド)
    file_save,Save情報
          RET,TRUE:正常終了,FALSE:エラー                                    */
gboolean
file_save_edit (FileSave *file_save)
{
  gboolean result;
  gint i;
#ifdef USE_THREAD
  GThread *id;
  id = g_thread_self ();
#else /* not USE_THREAD */
  gint id = THREAD_ID_STANDARD;
#endif /* not USE_THREAD */

  thread_insert (id, THREAD_MODE_SAVE, file_save->file);

  result = avi_save_with_options (file_save->avi_edit, file_save->file,
                        file_save->avi_save, file_save_edit_callback, NULL);
  g_free (file_save->file);
  for (i = 0; i < 2; i++)
    if (file_save->avi_edit[i])
      avi_release (file_save->avi_edit[i]);
  avi_save_free (file_save->avi_save);
  g_free (file_save);

  if (!result)
    while (thread_idling (id, -1));

  thread_delete (id);
  return result;
}

==> vmaid-1.8.25.tar.gz: vmaid-1.8.25/src/file.c <==
/*
    Video maid
    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "file.h"
#include "sigfile.h"
#include "sigmain.h"
#include "size.h"
#include "thread.h"
#include "misc/fileio.h"
#include "misc/misc.h"
#include "misc/misc_history.h"
#include "misc/misc_mdi.h"


/******************************************************************************
*                                                                             *
* ja:ファイル入力関数群                                                       *
*                                                                             *
******************************************************************************/
/*  ja:AVIファイルを開く
     avi,AVI編集構造体
    file,ファイル名                                                         */
static void
file_open_avi (AviEdit     *avi_edit[],
               const gchar *file)
{
  gint i;
  guint32 streams;
  AviEdit *avi_edit_tmp;

  avi_edit[0] = avi_edit[1] = NULL;
  streams = MAX (avi_main_streams (file), 1);
  for (i = 0; i < streams && (!avi_edit[0] || !avi_edit[1]); i++)
    if ((avi_edit_tmp = avi_open (file, i)))
      switch (avi_type (avi_edit_tmp))
        {
          case AVI_STREAM_VIDEO:
            if (!avi_edit[0])
              avi_edit[0] = avi_edit_tmp;
            else
              avi_release (avi_edit_tmp);
              break;
          case AVI_STREAM_AUDIO:
            if (!avi_edit[1])
              avi_edit[1] = avi_edit_tmp;
            else
              avi_release (avi_edit_tmp);
            break;
          default:
            avi_release (avi_edit_tmp);
        }
}


/*  ja:ファイルを開く(独立スレッド)
    file_open,Open情報
     RET,ウインドウ情報                                                     */
VmaidWindow *
file_open_edit (FileOpen *file_open)
{
  gchar *filename;
  gint i, page_num;
#ifdef USE_THREAD
  GThread *id;
#else /* not USE_THREAD */
  gint id = THREAD_ID_STANDARD;
#endif /* not USE_THREAD */
  GtkWidget *xbox, *vbox;
  VmaidWindow *vmaid;

#ifdef USE_THREAD
  id = g_thread_self();
#endif /* USE_THREAD */
  thread_insert (id, THREAD_MODE_OPEN, file_open ? file_open->file : NULL);

  vmaid = g_malloc (sizeof (VmaidWindow));
  vmaid->undo = vmaid->redo = NULL;
  vmaid->rate = default_rate;
  vmaid->scale = 1;
  vmaid->cursor.stream = 0;
  vmaid->cursor.frame = 0;
  vmaid->select.stream = -1;
  vmaid->select.frame = -1;
  vmaid->top = 0;
  vmaid->avi_edit[0] = NULL;
  vmaid->avi_edit[1] = NULL;
  vmaid->timer_id = 0;
  if (!file_open || !file_open->file)
    {
      static gint fcount = 0; /* ja:新規ファイルの数 */

      /* ja:新規 */
      filename = g_strdup_printf ("new%04d.avi", fcount++ % 1000);
    }
  else
    {
      gchar *format = NULL;
      gint j, value, max = 0;

      filename = fileio_get_full_path (file_open->file);
      file_open_avi (vmaid->avi_edit, file_open->file);
      if ((vmaid->avi_edit[0] || vmaid->avi_edit[1]) && file_open->merge)
        {
          gchar *name;

          /* ja:連番 */
          name = g_path_get_basename (file_open->file);
          for (i = g_strlen (name) - 1; i >= 0; i--)
            if ('0' <= name[i] && name[i] <= '9')
              break;
          if (i >= 0)
            {
              gchar *dir, *head, *foot;

              dir = g_path_get_dirname (file_open->file);
              head = g_strdup (name);
              foot = name + i + 1;
              for (j = i - 1; j >= 0; j--)
                if (head[j] < '0' || '9' < head[j])
                  break;
              misc_str_to_val (&value, head + j + 1, 10, FALSE);
              head[j + 1] = '\0';
              format = g_strdup_printf ("%s/%s%%0%dd%s",
                                                    dir, head, i - j, foot);
              g_free (dir);
              g_free (head);
              max = 1;
              while (j < i)
                {
                  max *= 10;
                  j++;
                }
            }
          g_free (name);
        }
      if (format)
        {
          gboolean result = TRUE;
          gchar *file;
          AviEdit *avi_edit_old[2], *avi_edit_tmp[2];

          /* ja:連番 */
          for (i = value - 1; i >= 0; i--)
            {
              /* ja:連番前 */
              file = g_strdup_printf (format, i);
              file_open_avi (avi_edit_tmp, file);
              g_free (file);
              if (!avi_edit_tmp[0] && !avi_edit_tmp[1])
                break;
              for (j = 0; j < 2; j++)
                if (vmaid->avi_edit[j])
                  {
                    avi_edit_old[j] = avi_clone (vmaid->avi_edit[j]);
                    if (!avi_paste (vmaid->avi_edit[j], 0, avi_edit_tmp[j]))
                      result = FALSE;
                    }
                  else
                    {
                      avi_edit_old[j] = NULL;
                      vmaid->avi_edit[j] = avi_edit_tmp[j];
                    }
              if (result)
                {
                  for (j = 0; j < 2; j++)
                    if (avi_edit_old[j])
                      avi_release (avi_edit_old[j]);
                }
              else
                {
                  for (j = 0; j < 2; j++)
                    {
                      if (vmaid->avi_edit[j])
                        avi_release (vmaid->avi_edit[j]);
                      vmaid->avi_edit[j] = avi_edit_old[j];
                    }
                  break;
                }
            }
          for (i = value + 1, result = TRUE; i < max && result; i++)
            {
              /* ja:連番後 */
              file = g_strdup_printf (format, i);
              file_open_avi (avi_edit_tmp, file);
              g_free (file);
              if (!avi_edit_tmp[0] && !avi_edit_tmp[1])
                break;
              for (j = 0; j < 2; j++)
                if (vmaid->avi_edit[j])
                  {
                    avi_edit_old[j] = avi_clone (vmaid->avi_edit[j]);
                    if (!avi_paste (vmaid->avi_edit[j],
                            avi_length (vmaid->avi_edit[j]), avi_edit_tmp[j]))
                      result=FALSE;
                  }
                else
                  {
                    avi_edit_old[j] = NULL;
                    vmaid->avi_edit[j] = avi_edit_tmp[j];
                  }
              if (result)
                {
                  for (j = 0; j < 2; j++)
                    if (avi_edit_old[j])
                      avi_release (avi_edit_old[j]);
                }
              else
                {
                  for (j = 0; j < 2; j++)
                    {
                      if (vmaid->avi_edit[j])
                        avi_release(vmaid->avi_edit[j]);
                      vmaid->avi_edit[j]=avi_edit_old[j];
                    }
                }
            }
        }
      if (!vmaid->avi_edit[0] && !vmaid->avi_edit[1]
                    && avi_file_type (file_open->file) == AVI_TYPE_SCENARIO)
        {
          gchar *scenario = NULL;
          goffset offset;
          AviEdit **avi_edit;
          FileIO *fio;

          /* ja:ファイルを開く */
          fio = fileio_open (file_open->file, FILEIO_ACCESS_READ,
                                FILEIO_SHARE_READ, FILEIO_MODE_OPEN_EXISTING);
          if (!fio)
            {
              g_free (file_open->file);
              g_free (file_open);
              g_free (vmaid);
              return NULL;
            }
          /* ja:読み込み */
          offset = fileio_seek (fio, 0, SEEK_END);
          if (offset == -1 || fileio_seek (fio, 0, FILEIO_SEEK_SET) == -1
                                || !(scenario = g_malloc ((gssize)offset + 1)))
            {
              fileio_close (fio);
              g_free (file_open->file);
              g_free (file_open);
              g_free (vmaid);
              return 0;
            }
          scenario[offset] = '\0';
          if (fileio_read (fio, scenario, (gssize)offset) != offset)
            {
              fileio_close (fio);
              g_free (scenario);
              g_free (file_open->file);
              g_free (file_open);
              g_free (vmaid);
              return 0;
            }
          if (!fileio_close (fio))
            {
              g_free (scenario);
              g_free (file_open->file);
              g_free (file_open);
              g_free (vmaid);
              return 0;
            }
          if ((avi_edit = avi_from_scenario (scenario)))
            for (i = 0; avi_edit[i]; i++)
              switch (avi_type (avi_edit[i]))
                {
                  case AVI_STREAM_VIDEO:
                    if (!vmaid->avi_edit[0])
                      vmaid->avi_edit[0] = avi_edit[i];
                    else
                      avi_release (avi_edit[i]);
                    break;
                  case AVI_STREAM_AUDIO:
                    if (!vmaid->avi_edit[1])
                      vmaid->avi_edit[1] = avi_edit[i];
                    else
                      avi_release (avi_edit[i]);
                    break;
                  default:
                    avi_release(avi_edit[i]);
                }
        }
    }

  if (!thread_idling (id, 100))
    {
      g_free (file_open->file);
      g_free (file_open);
      for (i = 0; i < 2; i++)
        if (vmaid->avi_edit[i])
          avi_release(vmaid->avi_edit[i]);
      g_free (vmaid);
      return NULL;
    }

  if (vmaid->avi_edit[0])
    {
      size_set_scale (default_view, &vmaid->width, &vmaid->height,
                                        avi_get_width (vmaid->avi_edit[0]),
                                        avi_get_height (vmaid->avi_edit[0]));
      vmaid->rate = avi_get_rate (vmaid->avi_edit[0]);
      vmaid->scale = avi_get_scale (vmaid->avi_edit[0]);
    }
  else
    {
      size_set_scale (default_view, &vmaid->width, &vmaid->height, -1, -1);
    }
  if (!vmaid->avi_edit[0] && vmaid->avi_edit[1])
    vmaid->cursor.stream = 1;

#ifdef USE_THREAD
  gdk_threads_enter();
#endif /* USE_THREAD */

  /* ja:表示エリア */
  vmaid->drawing = gtk_drawing_area_new ();
  g_signal_connect (G_OBJECT (vmaid->drawing), "focus-in-event",
                                G_CALLBACK (signal_focus_in), vmaid);
  g_signal_connect (G_OBJECT (vmaid->drawing), "focus-out-event",
                                G_CALLBACK (signal_focus_out), vmaid);
  g_signal_connect (G_OBJECT (vmaid->drawing), "realize",
                                G_CALLBACK (signal_realize), NULL);
  g_signal_connect (G_OBJECT (vmaid->drawing), "configure-event",
                                G_CALLBACK (signal_config), vmaid);
  g_signal_connect (G_OBJECT (vmaid->drawing), "expose-event",
                                G_CALLBACK (signal_expose), vmaid);
  g_signal_connect (G_OBJECT (vmaid->drawing), "button-press-event",
                                G_CALLBACK (signal_button_press_draw), vmaid);
  g_signal_connect (G_OBJECT (vmaid->drawing), "motion-notify-event",
                                G_CALLBACK (signal_motion_notify), vmaid);
  g_signal_connect (G_OBJECT (vmaid->drawing), "button-release-event",
                                G_CALLBACK (signal_button_release), vmaid);
  g_signal_connect (G_OBJECT (vmaid->drawing), "scroll-event",
                                G_CALLBACK (signal_scroll), vmaid);
  g_signal_connect_after (G_OBJECT (vmaid->drawing), "key-press-event",
                                G_CALLBACK (signal_key_press), vmaid);
  g_signal_connect (GTK_OBJECT (vmaid->drawing), "destroy",
                                G_CALLBACK (signal_destroy_draw), vmaid);
  gtk_widget_add_events (vmaid->drawing, GDK_POINTER_MOTION_MASK
                            | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
                            | GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK
                            | GDK_FOCUS_CHANGE_MASK);
  gtk_widget_set_usize (vmaid->drawing,
                vmaid->width * 3 + 4, system_font_height * 2
                + ((vmaid->avi_edit[0] ? 1 : 0) + (vmaid->avi_edit[1] ? 1 : 0))
                                                            * vmaid->height);
  GTK_WIDGET_SET_FLAGS (vmaid->drawing, GTK_CAN_FOCUS);
  /* ja:スクロールバー */
  vmaid->hscroll = gtk_hscrollbar_new
                    (GTK_ADJUSTMENT (gtk_adjustment_new (0, 0, 0, 1, 1, 1)));
  /* ja:ボックス */
  xbox = gtk_hbox_new (FALSE, 0);
  vbox = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), vmaid->drawing, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), vmaid->hscroll, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (xbox), vbox, FALSE, FALSE,0);
  /* ja:表示 */
  page_num = misc_mdi_append_page (MISC_MDI (mdi),
                        xbox, filename, !file_open || !file_open->file, vmaid);
  gtk_widget_show_all (xbox);
  gtk_notebook_set_page (GTK_NOTEBOOK (mdi), page_num);
  gtk_widget_grab_focus (vmaid->drawing);

  /* ja:新規作成ではなく、ファイルが存在する時には履歴に加える */
  if (file_open && fileio_isfile (file_open->file))
    misc_history_add_file (MISC_HISTORY (history), file_open->file);

#ifdef USE_THREAD
  gdk_threads_leave();
#endif /* USE_THREAD */

  if (file_open)
    {
      g_free (file_open->file);
      g_free (file_open);
    }
  g_free (filename);
  thread_delete (id);

  return vmaid;
}


/******************************************************************************
*                                                                             *
* ja:ファイル出力関数群                                                       *
*                                                                             *
******************************************************************************/
static gboolean
file_save_edit_callback (gint percent, gpointer user_data)
{
#ifdef USE_THREAD
  return thread_idling (g_thread_self (), percent);
#else /* not USE_THREAD */
  return thread_idling (THREAD_ID_STANDARD, percent);
#endif /* not USE_THREAD */
}


/*  ja:AVIファイルを保存する(独立スレッド)
    file_save,Save情報
          RET,TRUE:正常終了,FALSE:エラー                                    */
gboolean
file_save_edit (FileSave *file_save)
{
  gboolean result;
  gint i;
#ifdef USE_THREAD
  GThread *id;
  id = g_thread_self ();
#else /* not USE_THREAD */
  gint id = THREAD_ID_STANDARD;
#endif /* not USE_THREAD */

  thread_insert (id, THREAD_MODE_SAVE, file_save->file);

  result = avi_save_with_options (file_save->avi_edit, file_save->file,
                        file_save->avi_save, file_save_edit_callback, NULL);
  g_free (file_save->file);
  for (i = 0; i < 2; i++)
    if (file_save->avi_edit[i])
      avi_release (file_save->avi_edit[i]);
  avi_save_free (file_save->avi_save);
  g_free (file_save);

  if (!result)
    while (thread_idling (id, -1));

  thread_delete (id);
  return result;
}
